<?php
defined('_JEXEC') or die();

require_once(JPATH_COMPONENT_SITE.DS.'configuration.php');
require_once(JPATH_COMPONENT_SITE.DS.'utils.form2content.php');

jimport('joomla.application.component.model');

class Form2ContentModelForms extends JModel
{
	var $_data = null;
	var $_pagination = null;
	var $_total = null;
	var $_context = 'com_form2content.viewforms';
	
	function __construct()
	{
		parent::__construct();
		
		$app 		=& JFactory::getApplication();		
		$limit 		= $app->getUserStateFromRequest('global.list.limit', 'limit', $app->getCfg('list_limit'), 'int');
		$limitstart = $app->getUserStateFromRequest($this->_context.'.forms.limitstart', 'limitstart', 0, 'int');
		
		// In case limit has been changed, adjust it
		$limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);
		
		$this->setState('limit', $limit);
		$this->setState('limitstart', $limitstart);				
	}

	function _cleanCache()
	{
		$cache = &JFactory::getCache('com_form2content');
		$cache->clean();		
	}

	function _buildQuery()
	{
		$query = 'SELECT a.*, c.state, c.publish_up as publish_up_c, c.publish_down as publish_down_c, ' .
				 'u.username as creator, p.title as projecttitle, sec.title AS section_name, cc.title AS name ' .
				 'FROM #__f2c_form a ' . 
				 'LEFT JOIN #__content c ON a.reference_id = c.id ' .
				 'LEFT JOIN #__f2c_project p ON a.projectid = p.id ' .
				 'LEFT JOIN #__users u ON a.created_by = u.id ' .
				 'LEFT JOIN #__sections sec ON a.sectionid = sec.id ' .
				 'LEFT JOIN #__categories AS cc ON a.catid = cc.id ' .
				 $this->_buildContentWhere() .
				 $this->_buildContentOrderBy();

		return $query;
	}

	function getData()
	{
		if(empty($this->_data))
		{
			$query = $this->_buildQuery();
			$this->_data = $this->_getList($query, $this->getState('limitstart'), $this->getState('limit'));
		}

		return $this->_data;
	}
	
	function getPagination()
	{
		if(empty($this->_pagination))
		{
			jimport('joomla.html.pagination');
			
			$this->_pagination = new JPagination($this->getTotal(), $this->getState('limitstart'), $this->getState('limit'));
		}
		
		return $this->_pagination;
	}
	
	function getTotal()
	{		
		if(empty($this->_total))
		{
			$query = $this->_buildQuery();
			$this->_total = $this->_getListCount($query);			
		}
		
		return $this->_total;		
	}	
	
	function _buildContentWhere()
	{
		$db					=& JFactory::getDBO();
		$app 				=& JFactory::getApplication();
		$filter_sectionid	= $app->getUserStateFromRequest($this->_context.'filter_sectionid', 'filter_sectionid', -1, 'int');
		$catid				= $app->getUserStateFromRequest($this->_context.'catid', 'catid', 0, 'int');
		$projectid			= $app->getUserStateFromRequest($this->_context.'projectid', 'projectid', -1, 'int');
		$search				= $app->getUserStateFromRequest($this->_context.'searchforms', 'search', '', 'string');
		$search				= JString::strtolower($search);
				
		$where = array();

		// Section filter
		if ($filter_sectionid >= 0) 
		{
			$where[] = 'a.sectionid = '.(int)$filter_sectionid;
		}

		// Category filter
		if ($catid > 0) 
		{
			$where[] = 'a.catid = '.(int)$catid;
		}
		// Content Type filter
		if($projectid > 0)
		{
			$where[] = 'projectid = ' . (int)$projectid;	
		}

		// Keyword filter
		if ($search) 
		{
			$where[] = 	'(LOWER(a.title) LIKE '.$db->Quote( '%'.$db->getEscaped( $search, true ).'%', false ) .
						' OR c.id = ' . (int) $search . ')';
		}
		
		$where = (count($where) ? ' WHERE '.implode(' AND ', $where) : '');

		return $where;
	}	
	
	function _buildContentOrderBy()
	{
		$app 				=& JFactory::getApplication();
		$filter_order		= $app->getUserStateFromRequest($this->_context.'filter_order', 'filter_order', 'section_name', 'cmd');
		$filter_order_Dir	= $app->getUserStateFromRequest($this->_context.'filter_order_Dir', 'filter_order_Dir', '', 'word');
		
		$orderby = ' ORDER BY '. $filter_order .' '. $filter_order_Dir .', cc.title, a.ordering ';
		return $orderby;
	}
			
	function saveorder($cid, $order)
	{
		$total		= count($cid);
		$conditions	= array();
		$categories = array();
		$config 	= new F2CConfig();
		
		JArrayHelper::toInteger($cid, array(0));
		JArrayHelper::toInteger($order, array(0));

		// Instantiate a form table object
		JTable::addIncludePath(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_form2content'.DS.'tables');
		$row =& JTable::getInstance('form', 'Table');

		// Update the ordering for items in the cid array
		for ($i = 0; $i < $total; $i ++)
		{
			$row->load((int)$cid[$i]);
			
			if ($row->ordering != $order[$i]) 
			{
				$row->ordering = $order[$i];
				
				if (!$row->store()) 
				{
					JError::raiseError( 500, $db->getErrorMsg());
					return false;
				}
				
				// remember to updateOrder this group
				$condition = 'catid = '.(int)$row->catid;
				$found = false;

				if(!array_key_exists($row->catid,$categories))
				{
					$categories[] = $row->catid;
				}
				
				foreach($conditions as $cond)
				{
					if ($cond[1] == $condition) 
					{
						$found = true;
						break;
					}
				}
					
				if (!$found)
				{
					$conditions[] = array ($row->id, $condition);
				}
			}
		}

		// execute updateOrder for each group
		foreach($conditions as $cond)
		{
			$row->load($cond[0]);
			$row->reorder($cond[1]);
		}

		if( count($categories) && 
			array_key_exists('autosync_article_order',get_object_vars($config)) && 
			$config->autosync_article_order)
		{
			foreach($categories as $category) 
			{
				ContentHelper::syncArticleOrder($category);
			}	
		}

		$this->_cleanCache();

		return true;
	}				
}
?>
