<?php
defined('_JEXEC') or die('Restricted acccess');

require_once(JPATH_COMPONENT_SITE.DS.'class.form2content.php');
require_once(JPATH_COMPONENT_SITE.DS.'shared.form2content.php');
require_once(JPATH_COMPONENT_SITE.DS.'utils.form2content.php');
require_once(JPATH_COMPONENT_SITE.DS.'storage.form2content.php');
require_once(JPATH_COMPONENT_SITE.DS.'validations.form2content.php');
require_once(JPATH_COMPONENT_SITE.DS.'configuration.php');

jimport('joomla.application.component.model');

class Form2ContentModelFormBase extends JModel
{
	var $_id = null;
	var $_contentTypeId = null;
	var $_contentTypeSettings = null;
	var $_contentType = null;
	var $_data = null;
	var $_fieldData = null;
	var $_dispatcher = null;
	
	function __construct()
	{
		$this->_dispatcher =& JDispatcher::getInstance();
		parent::__construct();
	}
	
	function setId($id = 0)
	{
		$this->_id = $id;
		$this->_data = null;
	}
	
	function isNew()
	{
		return ($this->_id == 0);
	}
	
	function &getData()
	{
		if (!$this->_loadData())
		{
			$this->_initData();	
		}

		$this->_loadContentTypeSettings($this->_contentTypeId);
		
		return $this->_data;		
	}
	
	function delete()
	{
		$cids = JRequest::getVar( 'cid', array(0), 'post', 'array');

		if (count($cids))
		{			
			foreach($cids as $cid) 
			{	
				$this->deleteSingleForm($cid);
			}
			
			$this->_cleanCache();
		}

		return true;
	}
	
	function deleteSingleForm($id)
	{
		JPluginHelper::importPlugin('form2content');
		
		$eventData 	= new F2C_EventArgs('delete');
		$this->setId($id);
		$this->getData();
				
		$eventData->form =& $this->_data;
		$eventData->fields =& $this->_fieldData;
		
		$result = $this->_dispatcher->trigger('onBeforeF2cContentDelete', array($eventData));		
		
		if($this->_data->reference_id)
		{
			// Trash the Joomla article
			$query = 'UPDATE #__content SET state = -2, ordering = 0 ' .
					 'WHERE id=' . $this->_data->reference_id;
			$this->_db->setQuery($query);
			
			if (!$this->_db->query()) 
			{
				$this->setError($this->_db->getErrorMsg());
				return false;
			}
		}
		
		$contentTypeId = $this->_data->projectid;
		
		// Delete the article images
		recursiveRemoveDirectory(Path::Combine(F2C_Image::GetImagesRootPath(), "p$contentTypeId/f$id"));							
		// Delete the file uploads		
		recursiveRemoveDirectory(Path::Combine(F2C_FileUpload::GetFilesRootPath(), "c$contentTypeId/a$id"));
		// Delete the field contents		
		deleteTableRows('#__f2c_fieldcontent', 'formid='.(int)$id);
		// Delete the form			
		$this->_db->setQuery('DELETE FROM #__f2c_form WHERE id='.(int)$id);
			
		if(!$this->_db->query())
		{
			$this->setError($this->_db->getErrorMsg());
			return false;
		}
		
		$result = $this->_dispatcher->trigger('onAfterF2cContentDelete', array($eventData));
		
		return true;									
	}
	
	function display()
	{
	}	

	function store()
	{
		$row =& $this->getTable();
		
		// Bind the form fields to the table
		if (!$row->bind($this->_data)) 
		{
			$this->setError($this->_db->getErrorMsg());
			return false;
		}
		
		if (!$row->check()) 
		{
			$this->setError($this->_db->getErrorMsg());
			return false;
		}

		// Store the table to the database
		if (!$row->store()) 
		{
			$this->setError($this->_db->getErrorMsg());
			return false;
		}
		
		return true;
	}
	
	function parse($publishState)
	{
		JPluginHelper::importPlugin('form2content');
		
		$eventData 	= new F2C_EventArgs('parse');

		$eventData->form =& $this->_data;
		$eventData->fields =& $this->_fieldData;

		$performParse = true;
		
		$result = $this->_dispatcher->trigger('onBeforeF2cContentParse', array($eventData, &$performParse));		

		if($performParse)
		{	
			$parseResult = parseSingleForm($this, $publishState);
	
			if(!$parseResult->success)
			{
				$this->setError(JText::_('FORM_ID') . ' ' . $this->_data->id . ': ' . $parseResult->message);
				return false;
			}
	
			$result = $this->_dispatcher->trigger('onAfterF2cContentParse', array($eventData));		
		}
				
		return true;
	}

	function copy()
	{
		$cids 					= JRequest::getVar('cid', array(0), 'post', 'array');
		$formTable 				= JTable::getInstance('form','Table');
		$fieldContentRow 		=& $this->getTable('FieldContent');
		$dateNow 				=& JFactory::getDate();
		$timestamp 				= $dateNow->toMySQL();
		$fileFieldTypes			= array();
		$config 				= new F2CConfig();
		
		JPluginHelper::importPlugin('form2content');
		
		foreach($cids as $cid)
		{
			// load the form
			$this->setId($cid);
			$this->_loadData();
			
			// get all the field definitions for this form
		 	$this->_db->setQuery("SELECT f.* FROM #__f2c_projectfields f "
		 						. "WHERE f.projectid =" . $this->_data->projectid
		 						. " ORDER BY ordering ASC");
		  	$fields = $this->_db->loadObjectList();	
			
			// convert the form to a saveable structure...
			foreach($this->_fieldData as $fieldData)
			{
				$elementName = 't'.$fieldData->id;
				$data[$elementName.'_fieldid'] = '';
				
				switch($fieldData->fieldtypeid)
				{					
					case F2C_FIELDTYPE_SINGLELINE:
					case F2C_FIELDTYPE_MULTILINETEXT:
					case F2C_FIELDTYPE_MULTILINEEDITOR:
					case F2C_FIELDTYPE_CHECKBOX:
					case F2C_FIELDTYPE_SINGLESELECTLIST:								
					case F2C_FIELDTYPE_DATEPICKER:
					case F2C_FIELDTYPE_DATABASE_LOOKUP:
						$data[$elementName] = $fieldData->attributes['VALUE'];
						break;
						
					case F2C_FIELDTYPE_IMAGE:
						if($fieldData->attributes['VALUE'])
						{
							$image 									= unserialize($fieldData->attributes['VALUE']);
							$imagePath 								= Path::Combine(F2C_Image::GetImagesRootPath(), 'p' . $this->_data->projectid);				
							$imagePath 								= Path::Combine($imagePath, 'f' . $this->_data->id);				
							$thumbsPath 							= Path::Combine($imagePath, 'thumbs');
							$imageFileName 							= F2C_Image::CreateFullImageName($image->filename, $fieldData->id);
							$data[$elementName.'_method'] 			= 'copy';
							$data[$elementName.'_del'] 				= null;
							$data[$elementName.'_filename'] 		= $image->filename;
							$data[$elementName.'_alt'] 				= $image->alt;
							$data[$elementName.'_title'] 			= $image->title;
							$data[$elementName.'_location'] 		= Path::Combine($imagePath, $imageFileName);;
							$data[$elementName.'_thumblocation'] 	= Path::Combine($thumbsPath, F2C_Image::CreateThumbnailImageName($image->filename, $fieldData->id));												
						}						
						break;
						
					case F2C_FIELDTYPE_IFRAME:
						if($fieldData->attributes['VALUE'])
						{
							$iframe = unserialize($fieldData->attributes['VALUE']);
							$data[$elementName] 			= $iframe->url;
							$data[$elementName.'_width'] 	= $iframe->width;
							$data[$elementName.'_height'] 	= $iframe->height;	
						}
						break;
						
					case F2C_FIELDTYPE_EMAIL:
						if($fieldData->attributes['VALUE'])
						{
							$email = unserialize($fieldData->attributes['VALUE']);
							$data[$elementName] 			= $email->email;
							$data[$elementName.'_display'] 	= $email->display;
						}
						break;
						
					case F2C_FIELDTYPE_HYPERLINK:
						if($fieldData->attributes['VALUE'])
						{
							$hyperlink = unserialize($fieldData->attributes['VALUE']);
							$data[$elementName] 			= $hyperlink->url;
							$data[$elementName.'_display'] 	= $hyperlink->display;
							$data[$elementName.'_title'] 	= $hyperlink->title;	
							$data[$elementName.'_target'] 	= $hyperlink->target;	
						}					
						break;
						
					case F2C_FIELDTYPE_INFOTEXT:
						// no action required
						break;
						
					case F2C_FIELDTYPE_DISPLAYLIST:
					
						if($fieldData->attributes['VALUE'])
						{
							$list = unserialize($fieldData->attributes['VALUE']);
							$rowKey = array();
							$keyCount = 0;
							
							foreach($list as $listItem)
							{
								$keyValue = $elementName.'_'.$keyCount;
								$rowKey[] = $keyValue;
								$data[$elementName.'_row'.$keyValue] = $listItem;
								$keyCount++;
							}
							
							$data[$elementName.'_rowkey'] = $rowKey;
						}
						break;
						
					case F2C_FIELDTYPE_MULTISELECTLIST:
						$value = $fieldData->attributes['VALUE'];
						
						if($value)
						{
							$data[$elementName] = unserialize($value);
						}
						break;	
						
					case F2C_FIELDTYPE_FILE:
						if($fieldData->attributes['VALUE'])
						{
							$filename 						= $fieldData->attributes['VALUE'];							
							$data[$elementName.'_method'] 	= 'copy';
							$data[$elementName.'_del'] 		= null;						
							$data[$elementName.'_location'] = Path::Combine(F2C_FileUpload::GetFilePath($this->_data->projectid, $this->_data->id, $fieldData->id), $filename);
							$data[$elementName.'_filename'] = $filename;
						}
						break;
						
					case F2C_FIELDTYPE_GEOCODER:
						$data[$elementName.'_addressid']	= null;
						$data[$elementName.'_address'] 		= $fieldData->attributes['ADDRESS'];		
						$data[$elementName.'_latid']		= null;
						$data[$elementName.'_lat'] 			= $fieldData->attributes['LAT'];
						$data[$elementName.'_lonid']		= null;
						$data[$elementName.'_lon'] 			= $fieldData->attributes['LON'];								
						break;
						
					case F2C_FIELDTYPE_DB_LOOKUP_MULTI:
					
						if(count($fieldData->attributes))
						{
							$valueList 	= $fieldData->attributes['VALUE'];
							$rowKey 	= array();
							$keyCount 	= 0;
							
							foreach($valueList as $listItem)
							{
								$keyValue = $elementName.'_'.$keyCount;
								$rowKey[] = $keyValue;
								$data[$elementName.'_row'.$keyValue] = $listItem;
								$keyCount++;
							}
							
							$data[$elementName.'_rowkey'] = $rowKey;
						}
						break;												
				}
			}
			
			if(!$formTable->load($cid))
			{
				$this->setError($formTable->getError());
				return false;
			}

			$formTable->title = JText::_('COPY_OF') . ' ' . $formTable->title;
			$formTable->title_alias = '';
			$formTable->id = 0; // force insert
			$formTable->created = $timestamp;
			$formTable->modified = $timestamp;
			$formTable->state = 0; // default unpublished
			$formTable->reference_id = null; // no article yet
			
			$eventData 			= new F2C_EventArgs('copy');
			$eventData->form 	=& $formTable;
			$eventData->fields 	=& $data;
			
			// This is always a new form
			$result = $this->_dispatcher->trigger('onBeforeF2cContentSave', array(&$eventData, true));
			
			if(!$formTable->store())
			{
				$this->setError($formTable->getError());
				return false;
			}
			
			// Update item order
			$formTable->reorder('catid = '.(int)$formTable->catid);
			
			$storage = new F2C_Storage($config, $formTable->id, true);
			$storage->preparedData = $data;
			$storage->storeFields($fields);
			
			// Sync stored data with model
			$this->setId($formTable->id);			
			$this->_loadData();
			
			$eventData = new F2C_EventArgs('copy');
			$eventData->form =& $this->_data;
			$eventData->fields =& $this->_fieldData;
			
			$result = $this->_dispatcher->trigger('onAfterF2cContentSave', array($eventData));
		}

		return true;
	}
		
	function _cleanCache()
	{
		$cache = &JFactory::getCache('com_form2content');
		$cache->clean();		
	}		

	function _loadContentTypeSettings($contentTypeId)
	{
		$this->_db->setQuery('SELECT * FROM #__f2c_project WHERE id=' . (int)$contentTypeId);			
		$this->_contentType = $this->_db->loadObject();
		$this->_contentTypeSettings = ($this->_contentType->settings) ? unserialize($this->_contentType->settings) : new F2C_ContentTypeSettings();
	}

	function _initForm($contentTypeId)
	{
		$this->_loadContentTypeSettings($contentTypeId);
		$user = JFactory::getUser();
		$createdate =& JFactory::getDate();
				
		$this->_data					= new stdClass();
		$this->_data->id				= 0;
		$this->_data->projectid			= $contentTypeId;
		$this->_data->title 			= ($this->_contentTypeSettings->title_default) ? $this->_contentTypeSettings->title_default : '';
		$this->_data->title_alias		= null;
		$this->_data->created_by		= $user->id;
		$this->_data->created_by_alias	= '';
		$this->_data->created			= $createdate->toUnix();;
		$this->_data->modified			= null;
		$this->_data->metakey 			= ($this->_contentTypeSettings->metakey_default) ? $this->_contentTypeSettings->metakey_default : '';
		$this->_data->metadesc 			= ($this->_contentTypeSettings->metadesc_default) ? $this->_contentTypeSettings->metadesc_default : '';
		$this->_data->sectionid			= ($this->_contentType->sectionid) ? $this->_contentType->sectionid : 0;
		$this->_data->catid				= ($this->_contentType->catid) ? $this->_contentType->catid : 0;
		$this->_data->intro_template	= ($this->_contentType->intro_template) ? $this->_contentType->intro_template : '';
		$this->_data->main_template		= ($this->_contentType->main_template) ? $this->_contentType->main_template : '';
		$this->_data->reference_id		= 0;
		$this->_data->ordering 			= 0;
		$this->_data->publish_up		= $createdate->toUnix();;
		$this->_data->publish_down		= null;
		$this->_data->state				= ($this->_contentTypeSettings->state_default) ? $this->_contentTypeSettings->state_default : 0;
		$this->_data->frontpage			= ($this->_contentTypeSettings->frontpage_default) ? $this->_contentTypeSettings->frontpage_default : 0;
		$this->_data->access			= ($this->_contentTypeSettings->access_default) ? $this->_contentTypeSettings->access_default : 0;
		$this->_data->attribs			= $this->_contentType->attribs;
		$this->_data->metadata			= $this->_contentType->metadata;			
	}		
}
?>