<?php
defined('_JEXEC') or die();

jimport('joomla.html.pagination');

require_once(JPATH_COMPONENT_SITE.DS.'configuration.php');
require_once(JPATH_COMPONENT_SITE.DS.'utils.form2content.php');

jimport('joomla.application.component.model');

class Form2ContentModelForms extends JModel
{
	var $_data = null;
	var $_pagination = null;
	var $_total = null;
	var $_context = 'com_form2content.viewforms';

	function __construct()
	{
		$config =& JFactory::getConfig();
		$app 	=& JFactory::getApplication();
		$option = JRequest::getCmd('option');
		
		parent::__construct();

		// Get the pagination request variables
		$this->setState('limit', $app->getUserStateFromRequest($option.'.limit', 'limit', $config->getValue('config.list_limit'), 'int'));
		$this->setState('limitstart', JRequest::getVar('limitstart', 0, '', 'int'));
		
		// In case limit has been changed, adjust limitstart accordingly
		$this->setState('limitstart', ($this->getState('limit') != 0 ? (floor($this->getState('limitstart') / $this->getState('limit')) * $this->getState('limit')) : 0));
	}

	function _cleanCache()
	{
		$cache = &JFactory::getCache('com_form2content');
		$cache->clean();		
	}
	
	function _buildQuery()
	{
		$query = 'SELECT a.*, c.state, c.publish_up as publish_up_c, c.publish_down as publish_down_c,' .
				 ' u.username as creator, p.title as projecttitle, sec.title AS section_name, cc.title AS name ' .
				 'FROM #__f2c_form a ' . 
				 'LEFT JOIN #__content c ON a.reference_id = c.id ' .
				 'LEFT JOIN #__f2c_project p ON a.projectid = p.id ' .
				 'LEFT JOIN #__users u ON a.created_by = u.id ' .
				 'LEFT JOIN #__sections sec ON a.sectionid = sec.id ' .
				 'LEFT JOIN #__categories AS cc ON a.catid = cc.id ' .
				$this->_buildContentWhere() . 
    			$this->_buildContentOrderBy();
    			
		return $query;
	}

	function getData()
	{
		if(empty($this->_data))
		{
			$query = $this->_buildQuery();						
			$this->_data = $this->_getList($query, $this->getState('limitstart'), $this->getState('limit'));
		}

		return $this->_data;
	}
	
	function getPagination()
	{
		if(empty($this->_pagination))
		{
			$this->_pagination = new JPaginationClient($this->getTotal(), $this->getState('limitstart'), $this->getState('limit'));
		}
		
		return $this->_pagination;
	}
	
	function getTotal()
	{		
		if(empty($this->_total))
		{
			$query = $this->_buildQuery();
			$this->_total = $this->_getListCount($query);			
		}
		
		return $this->_total;		
	}	
	
	function _buildContentWhere()
	{
		$app 				=& JFactory::getApplication();
		$option 			= JRequest::getCmd('option');		
		$db					=& JFactory::getDBO();
		$params 			=& $app->getParams();
		$config				= new F2CConfig();
		$user 				=& JFactory::getUser();
		$search				= $app->getUserStateFromRequest($option.'searchforms', 'searchforms', '', 'string');
		$search				= JString::strtolower($search);
		$filter_sectionid	= $app->getUserStateFromRequest($option.'filter_sectionid', 'filter_sectionid', -1, 'int');
		$catid				= $app->getUserStateFromRequest($option.'catid', 'catid', 0, 'int');
		$menuitemid 		= JRequest::getInt('Itemid');
		
		if ($menuitemid)
		{
			$menu = JSite::getMenu();
			$menuparams = $menu->getParams($menuitemid);
			$params->merge($menuparams);
		}		
		
		if($params->get('contenttypeid', 0) == 0)
		{
			// get content type from filterdropdown
			$projectid	= $app->getUserStateFromRequest($option.'projectid', 'projectid', -1, 'int');
		}
		else
		{
			// get content type from menu parameter
			$projectid = $params->get('contenttypeid', -1);
		}		
		
		$where = array();

		switch($config->edit_items_level)
		{
			case 0: // edit all items
				break;
			case 1: // edit own items only
				$where[] = 'a.created_by = ' . $user->id . ' ';
				break;
			case 2: // admins can edit all items
				$user = JFactory::getUser();
				if(!($user->usertype == 'Super Administrator' || $user->usertype == 'Administrator'))
				{
					$where[] = 'a.created_by = ' . $user->id . ' ';
				}
		}

		if($params->get('restrict_sectioncategory_id', '-1_-1') != '-1_-1')
		{
			// restrict section / category filter;
			$ids = explode('_', $params->get('restrict_sectioncategory_id', '-1_-1'));
			$filter_sectionid = $ids[0];
			$catid = $ids[1];
		}
		
		if ($filter_sectionid >= 0) 
		{
			$where[] = 'a.sectionid = '.(int)$filter_sectionid;
		}

		if ($catid > 0) 
		{
			$where[] = 'a.catid = '.(int)$catid;
		}
		
		if ($search) 
		{
			$where[] = 'LOWER(a.title) LIKE '.$db->Quote( '%'.$db->getEscaped( $search, true ).'%', false );
		}

		if($projectid > 0)
		{
			$where[] = ' projectid = ' . $projectid . ' ';	
		}
		
		//only published projects in front-end
		$where[] = ' p.published = 1 ';
		
		$where = (count($where) ? ' WHERE '. implode( ' AND ', $where ) : '' );

		return $where;
	}
	
	function _buildContentOrderBy()
	{
		$app 				=& JFactory::getApplication();
		$option 			= JRequest::getCmd('option');		
		$filter_order		= $app->getUserStateFromRequest($this->_context.'filter_order', 'filter_order', 'section_name', 'cmd');
		$filter_order_Dir	= $app->getUserStateFromRequest($this->_context.'filter_order_Dir', 'filter_order_Dir', '', 'word');

		$orderby = ' ORDER BY '. $filter_order .' '. $filter_order_Dir .', cc.title, a.ordering ';

		return $orderby;
	}
	
	function saveorder($cid, $order)
	{
		$total		= count($cid);
		$conditions	= array();
		$categories = array();
		$config 	= new F2CConfig();
		
		JArrayHelper::toInteger($cid, array(0));
		JArrayHelper::toInteger($order, array(0));

		// Instantiate a form table object
		JTable::addIncludePath(JPATH_SITE.DS.'components'.DS.'com_form2content'.DS.'tables');
		$row =& JTable::getInstance('form', 'Table');

		// Update the ordering for items in the cid array
		for ($i = 0; $i < $total; $i ++)
		{
			$row->load((int)$cid[$i]);
			
			if ($row->ordering != $order[$i]) 
			{
				$row->ordering = $order[$i];
				
				if (!$row->store()) 
				{
					JError::raiseError( 500, $db->getErrorMsg());
					return false;
				}
				
				// remember to updateOrder this group
				$condition = 'catid = '.(int)$row->catid;
				$found = false;

				if(!array_key_exists($row->catid,$categories))
				{
					$categories[] = $row->catid;
				}
				
				foreach($conditions as $cond)
				{
					if ($cond[1] == $condition) 
					{
						$found = true;
						break;
					}
				}
					
				if (!$found)
				{
					$conditions[] = array ($row->id, $condition);
				}
			}
		}

		// execute updateOrder for each group
		foreach($conditions as $cond)
		{
			$row->load($cond[0]);
			$row->reorder($cond[1]);
		}
		
		if( count($categories) && 
			array_key_exists('autosync_article_order',get_object_vars($config)) && 
			$config->autosync_article_order)
		{
			foreach($categories as $category) 
			{
				ContentHelper::syncArticleOrder($category);
			}	
		}

		$this->_cleanCache();
		return true;
	}				
}
?>
