<?php
defined('_JEXEC') or die('Restricted acccess');

require_once(JPATH_SITE.DS.'components'.DS.'com_form2content'.DS.'configuration.php');
require_once(JPATH_SITE.DS.'components'.DS.'com_form2content'.DS.'class.form2content.php');
require_once(JPATH_SITE.DS.'components'.DS.'com_form2content'.DS.'utils.form2content.php');
require_once(JPATH_SITE.DS.'components'.DS.'com_content'.DS.'helpers'.DS.'route.php');

defined('F2C_ENGINE_UNDEFINED')		or define('F2C_ENGINE_UNDEFINED', -1);
defined('F2C_ENGINE_PATTEMPLATE')	or define('F2C_ENGINE_PATTEMPLATE', 0);
defined('F2C_ENGINE_SMARTY')		or define('F2C_ENGINE_SMARTY', 1);

defined('F2C_TEMPLATE_INTRO')		or define('F2C_TEMPLATE_INTRO', 0);
defined('F2C_TEMPLATE_MAIN')		or define('F2C_TEMPLATE_MAIN', 1);

class F2C_Parser
{
	var $_error = '';
	var $_engineType = F2C_ENGINE_UNDEFINED;
	var $_parser = null;
	var $_f2cConfig = null;
	
	function F2C_Parser()
	{
		$this->_f2cConfig = new F2CConfig();
	}
	
	function getError()
	{
		return $this->_error;
	}
	
	function setError($msg)
	{
		$this->_error = $msg;
	}
	
	function _detectEngine($templateFile)
	{
		$sample = JFile::Read($templateFile, false, 9);
		
		if(strtolower($sample) == '<mos:tmpl' ||
		   strtolower(substr($sample, 3)) == '<mos:t')
		{
			return F2C_ENGINE_PATTEMPLATE;
		}
		else
		{
			return F2C_ENGINE_SMARTY;
		}		
	}
	
	function addTemplate($templateName, $templateType)
	{
		$filename = Path::Combine(F2C_Config::getTemplatePath(), $templateName);
		
		if(!JFile::exists($filename))
		{
			$this->setError(JText::_('ERROR_TEMPLATE_NOT_FOUND'));
			return false;
		}

		$engineType = $this->_detectEngine($filename);
		
		if($templateType == F2C_TEMPLATE_INTRO)
		{
			$this->_engineType = $engineType;
		}
		else
		{
			if($this->_engineType != $engineType)
			{
				$this->SetError('Main template must be of same engine type as the intro template.');
				return false;
			}
		}		 

		if(!$this->_parser)
		{
			switch($this->_engineType)
			{
				case F2C_ENGINE_PATTEMPLATE:
					if (!class_exists('F2C_PatTemplate')) 
					{
						require_once(JPATH_SITE.DS.'components'.DS.'com_form2content'.DS.'pattemplate.form2content.php');
					}
					$this->_parser = new F2C_PatTemplate($this->_f2cConfig);					
					break;
					
				case F2C_ENGINE_SMARTY:
					if (!class_exists('F2C_Smarty')) 
					{
						require_once(JPATH_SITE.DS.'components'.DS.'com_form2content'.DS.'smarty.form2content.php');
					}					
					$this->_parser = new F2C_Smarty($this->_f2cConfig);
					break;
			}
		}

		if(!$this->_parser->addTemplate($templateName, $templateType))
		{
			$this->setError($this->_parser->error);
			return false;
		}
		
		return true;	
	}
	
	function clearTemplates()
	{
		$this->_parser->clearTemplates();
	}
	
	function addVar($name, $value)
	{
		$this->_parser->addVar($name, $value);
	}

	function clearVar($name)
	{
		$this->_parser->clearVar($name);
	}
	
	function addVars($form)
	{
		// Clear all previous vars
		$this->_parser->clearAllVars();
		
		$this->_parser->form = $form;
		
		$usrTmp 			= JFactory::getUser($form->created_by);
		$db 				=& JFactory::getDBO();
		$nullDate 			= $db->getNullDate();
		$joomlaId 			= $form->reference_id;
		$startPublishing 	= '';
		$finishPublishing 	= '';		
		$creationDate 		=  JHTML::_('date', $form->created, $this->_f2cConfig->date_format . ' %H:%M:%S');
		$modifiedDate 		=  ($form->modified != '' && $form->modified != $nullDate) ? JHTML::_('date', $form->modified, $this->_f2cConfig->date_format . ' %H:%M:%S') : '';
		
		if($form->publish_up != $nullDate)
		{
			$startPublishing = JHTML::_('date', $form->publish_up, $this->_f2cConfig->date_format . ' %H:%M:%S');
		}
	
		if($form->publish_down != $nullDate)
		{
			$finishPublishing = JHTML::_('date', $form->publish_down, $this->_f2cConfig->date_format . ' %H:%M:%S');
		}
		
		$slug = ($form->title_alias) ? $joomlaId.':'.$form->title_alias : $joomlaId;
		$catslug = ($form->catAlias) ? $form->catid.':'.$form->catAlias : $form->catid;
		
		$this->_parser->addVar('JOOMLA_ID', $joomlaId);
		$this->_parser->addVar('JOOMLA_ARTICLE_LINK', JRoute::_(ContentHelperRoute::getArticleRoute($slug, $catslug, $form->sectionid)));		 
		$this->_parser->addVar('JOOMLA_SECTION_ID', $form->sectionid);
		$this->_parser->addVar('JOOMLA_SECTION_TITLE', $form->secTitle);
		$this->_parser->addVar('JOOMLA_SECTION_ALIAS', $form->secAlias);
		$this->_parser->addVar('JOOMLA_CATEGORY_ID', $form->catid);
		$this->_parser->addVar('JOOMLA_CATEGORY_TITLE', $form->catTitle);
		$this->_parser->addVar('JOOMLA_CATEGORY_ALIAS', $form->catAlias);
		$this->_parser->addVar('JOOMLA_PUBLISH_UP', $startPublishing);
		$this->_parser->addVar('JOOMLA_PUBLISH_DOWN', $finishPublishing);	
		$this->_parser->addVar('JOOMLA_CREATED', $creationDate);	
		$this->_parser->addVar('JOOMLA_MODIFIED', $modifiedDate);			
		$this->_parser->addVar('JOOMLA_TITLE', HtmlHelper::stringHTMLSafe($form->title));		
		$this->_parser->addVar('JOOMLA_TITLE_ALIAS', HtmlHelper::stringHTMLSafe($form->title_alias));
		$this->_parser->addVar('JOOMLA_META_KEYWORDS', HtmlHelper::stringHTMLSafe($form->metakey));
		$this->_parser->addVar('JOOMLA_META_DESCRIPTION', HtmlHelper::stringHTMLSafe($form->metadesc));
		$this->_parser->addVar('JOOMLA_AUTHOR', HtmlHelper::stringHTMLSafe($usrTmp->name));
		$this->_parser->addVar('JOOMLA_AUTHOR_USERNAME', HtmlHelper::stringHTMLSafe($usrTmp->username));
		$this->_parser->addVar('JOOMLA_AUTHOR_EMAIL', $usrTmp->email);	
		$this->_parser->addVar('JOOMLA_AUTHOR_ID', $form->created_by);
		$this->_parser->addVar('JOOMLA_AUTHOR_ALIAS', $form->created_by_alias);	
		// Add F2C parameters to template
		$this->_parser->addVar('F2C_ID', $form->id);		
		$this->_parser->addVar('F2C_IMAGES_PATH_THUMBS_RELATIVE', F2C_Image::GetThumbnailsPath($form->projectid, $form->id, true)); 
		$this->_parser->addVar('F2C_IMAGES_PATH_THUMBS_ABSOLUTE', F2C_Image::GetThumbnailsPath($form->projectid, $form->id, false)); 
		$this->_parser->addVar('F2C_IMAGES_PATH_RELATIVE', F2C_Image::GetImagesPath($form->projectid, $form->id, true)); 
		$this->_parser->addVar('F2C_IMAGES_PATH_ABSOLUTE', F2C_Image::GetImagesPath($form->projectid, $form->id, false)); 
		
		foreach($form->fields as $field)
		{
			$this->_parser->addFormVar($field);
		}
		
		return true;
	}
	
	function parseIntro()
	{
		$content = $this->_parser->parseIntro();
		$this->setError($this->_parser->error);
		return $content;
	}

	function parseMain()
	{
		$content = $this->_parser->parseMain();
		$this->setError($this->_parser->error);
		return $content;
	}
	
	function getTemplateVars($formVars, &$usedVars)
	{
		$this->_parser->getTemplateVars($formVars, $usedVars);
	}
}
?>
