<?php
defined('_JEXEC') or die('Restricted acccess');

if (!class_exists('patFactory')) 
{
	require_once(JPATH_SITE.DS.'plugins'.DS.'system'.DS.'legacy'.DS.'patfactory.php');
}

class F2C_PatTemplate
{
	var $templates;
	var $config;
	var $form;
	var $error = '';
	
	function F2C_PatTemplate($config)
	{
		$this->config = $config;
	}

	function parseIntro()
	{
		$parsedContent = $this->templates[F2C_TEMPLATE_INTRO]->getParsedTemplate('root');
	
		if (patErrorManager::isError($parsedContent))
		{
			$this->error = $parsedIntroContent->message;
		}
		
		return $parsedContent;
	}

	function parseMain()
	{
		$parsedContent = '';
		
		if(array_key_exists(F2C_TEMPLATE_MAIN, $this->templates))
		{	
			$parsedContent = $this->templates[F2C_TEMPLATE_MAIN]->getParsedTemplate('root');
	
			if (patErrorManager::isError($parsedContent))
			{
				$this->error = $parsedIntroContent->message;
			}
		}
		
		return $parsedContent;
	}
	
	function addTemplate($templateName, $templateType)
	{
		if(!JFile::exists(Path::Combine(F2C_Config::getTemplatePath(), $templateName)))
		{
			$this->error = JText::_('ERROR_TEMPLATE_NOT_FOUND');
			return false;
		}
		
		$template =& patFactory::createTemplate( '', false, false );
		$template->freeAllTemplates();
		$template->setRoot(rtrim(F2C_Config::getTemplatePath(), "/\\"));	

		if(!$template->readTemplatesFromInput($templateName))
		{
			$this->error = JText::_('ERROR_UNKNOWN_PARSE_ERROR');
			return false;
		}
	
		$this->templates[$templateType] = $template;
		
		return true;	
	}
		
	function clearTemplates()
	{
		foreach($this->templates as $template)
		{
			$template->clearTemplate('root');
		}
	}
		
	function addVar($name, $value)
	{
		foreach($this->templates as $template)
		{
			$template->addGlobalVar($name, $value);
		}
	}
	
	function clearVar($name)
	{
		foreach($this->templates as $template)
		{
			$template->clearGlobalVar($name);
		}
	}
	
	function clearAllVars()
	{
		foreach($this->templates as $template)
		{
			$template->clearGlobalVars();
			$template->clearTemplate('root');
		}
	}
	
	function addFormVar($field)
	{
		$F2C_FIELD_FUNCTION_MAPPING = array(F2C_FIELDTYPE_SINGLELINE => 'SingleLineText',
									F2C_FIELDTYPE_MULTILINETEXT => 'MultiLineText',
									F2C_FIELDTYPE_MULTILINEEDITOR => 'MultiLineEditor',
									F2C_FIELDTYPE_CHECKBOX => 'CheckBox',
									F2C_FIELDTYPE_SINGLESELECTLIST => 'SingleSelectList', 
									F2C_FIELDTYPE_IMAGE => 'Image',
									F2C_FIELDTYPE_IFRAME => 'IFrame', 
									F2C_FIELDTYPE_EMAIL => 'Email', 
									F2C_FIELDTYPE_HYPERLINK => 'Hyperlink', 
									F2C_FIELDTYPE_MULTISELECTLIST => 'MultiSelectList', 
									F2C_FIELDTYPE_INFOTEXT => 'InfoText', 
									F2C_FIELDTYPE_DATEPICKER => 'DatePicker', 
									F2C_FIELDTYPE_DISPLAYLIST => 'DisplayList', 
									F2C_FIELDTYPE_FILE => 'FileUpload', 
									F2C_FIELDTYPE_DATABASE_LOOKUP => 'DatabaseLookup', 
									F2C_FIELDTYPE_GEOCODER => 'GeoCoder',
									F2C_FIELDTYPE_DB_LOOKUP_MULTI => 'DatabaseLookupMulti');

		$functionName 	= '_addVar'.$F2C_FIELD_FUNCTION_MAPPING[$field->fieldtypeid];
		$fieldData 		= count($field->attributes) ? $field->attributes : null;

		foreach($this->templates as $template)
		{
			// Leave the title for backward compatibility
			$template->addGlobalVar($field->fieldname.'_TITLE', HtmlHelper::stringHTMLSafe($field->title));				
			$template->addGlobalVar($field->fieldname.'_CAPTION', HtmlHelper::stringHTMLSafe($field->title));
			$template->addGlobalVar($field->fieldname.'_DESCRIPTION', HtmlHelper::stringHTMLSafe($field->description));			
			$this->$functionName($field, $fieldData, $template);
		}
	}
	
	function _addVarSingleLineText($field, $data, &$template)
	{
		$template->addGlobalVar($field->fieldname, HtmlHelper::stringHTMLSafe($data['VALUE']));
	}

	function _addVarMultiLineText($field, $data, &$template)
	{
		$template->addGlobalVar($field->fieldname, nl2br(HtmlHelper::stringHTMLSafe($data['VALUE'])));
	}
	
	function _addVarMultiLineEditor($field, $data, &$template)
	{
		$template->addGlobalVar($field->fieldname, $data['VALUE']);
	}
	
	function _addVarCheckBox($field, $data, &$template)
	{
		if($data['VALUE'])
		{
			$template->addGlobalVar($field->fieldname, $data['VALUE']); 
		}
	}

	function _addVarSingleSelectList($field, $data, &$template)
	{
		if($data['VALUE'])
		{
			$fieldText 	= '';
			$settings 	= unserialize($field->settings);

			if(array_key_exists($data['VALUE'], $settings->listOptions))
			{
				$fieldText = $settings->listOptions[$data['VALUE']];
			}
								
			$template->addGlobalVar($field->fieldname, $data['VALUE']);
			$template->addGlobalVar($field->fieldname.'_TEXT', $fieldText);
		}
	}
	
	function _addVarImage($field, $data, &$template)
	{
		if($data['VALUE'])
		{
			$settings 	= unserialize($field->settings);			
			$image 		= unserialize($data['VALUE']);
			
			if($image->filename)
			{
				$tagWidth 	= ($image->width > 0) ? ' width="'.$image->width.'"' : '';
				$tagHeight 	= ($image->height > 0) ? ' height="'.$image->height.'"' : '';
				
				if($settings->imageOutputMode == 0)
				{				
					$template->addGlobalVar($field->fieldname, F2C_Image::GetImagesUrl($this->form->projectid, $this->form->id) . $image->filename);						
				}
				else
				{
					$template->addGlobalVar($field->fieldname, '<img src="' . F2C_Image::GetImagesUrl($this->form->projectid, $this->form->id) . $image->filename . '" alt="' . $image->alt . '" title="' . $image->title . '"' . $tagWidth . $tagHeight . '/>');
				}
				
				// add image information
				$template->addGlobalVar($field->fieldname.'_IMG', '<img src="' . F2C_Image::GetImagesUrl($this->form->projectid, $this->form->id) . $image->filename . '" alt="' . $image->alt . '" title="' . $image->title . '"' . $tagWidth . $tagHeight . '/>');
				$template->addGlobalVar($field->fieldname.'_RAW', F2C_Image::GetImagesUrl($this->form->projectid, $this->form->id) . $image->filename);					
				$template->addGlobalVar($field->fieldname.'_ALT', HtmlHelper::stringHTMLSafe($image->alt));					
				$template->addGlobalVar($field->fieldname.'_TITLE', HtmlHelper::stringHTMLSafe($image->title));					
				$template->addGlobalVar($field->fieldname.'_WIDTH', ($image->width > 0) ? $image->width : '');					
				$template->addGlobalVar($field->fieldname.'_HEIGHT', ($image->height > 0) ? $image->height : '');					
				$template->addGlobalVar($field->fieldname.'_WIDTH_THUMB', ($image->widthThumbnail > 0) ? $image->widthThumbnail : '');					
				$template->addGlobalVar($field->fieldname.'_HEIGHT_THUMB', ($image->heightThumbnail > 0) ? $image->heightThumbnail : '');					
				
				// add image urls
				$template->addGlobalVar($field->fieldname.'_PATH_ABSOLUTE', Path::Combine(F2C_Image::GetImagesPath($this->form->projectid, $this->form->id, false), $image->filename));
				$template->addGlobalVar($field->fieldname.'_PATH_RELATIVE', Path::Combine(F2C_Image::GetImagesPath($this->form->projectid, $this->form->id, true), $image->filename));
				
				// add thumbnail urls
				$template->addGlobalVar($field->fieldname.'_THUMB_URL_ABSOLUTE', Path::Combine(F2C_Image::GetThumbnailsUrl($this->form->projectid, $this->form->id), $image->filename));
				$template->addGlobalVar($field->fieldname.'_THUMB_URL_RELATIVE', Path::Combine(F2C_Image::GetThumbnailsUrl($this->form->projectid, $this->form->id, true), $image->filename));
			}
			else
			{
				// no image was specified
				$template->addGlobalVar($field->fieldname, '');
				$template->addGlobalVar($field->fieldname.'_IMG', '');
				$template->addGlobalVar($field->fieldname.'_RAW', '');
				$template->addGlobalVar($field->fieldname.'_PATH_ABSOLUTE', '');
				$template->addGlobalVar($field->fieldname.'_PATH_RELATIVE', '');
				$template->addGlobalVar($field->fieldname.'_THUMB_URL_ABSOLUTE', '');
				$template->addGlobalVar($field->fieldname.'_THUMB_URL_RELATIVE', '');
				$template->addGlobalVar($field->fieldname.'_ALT', '');					
				$template->addGlobalVar($field->fieldname.'_TITLE', '');					
			} 
		}
	}

	function _addVarIFrame($field, $data, &$template)
	{
		if($data['VALUE'])
		{
			$iframe = unserialize($data['VALUE']);
			$iframeTag = '<iframe src="' . $iframe->url . '" height="' . $iframe->height . '" width="' . $iframe->width . '"></iframe>';
			$template->addGlobalVar($field->fieldname, $iframeTag);
		}				
	}	
	
	function _addVarEmail($field, $data, &$template)
	{
		$emailTag = '';
		$emailAddress = '';
		$emailDisplay = '';
		
		if($data['VALUE'])
		{
			$email = unserialize($data['VALUE']); 
			
			if($email->email)
			{
				$display = $email->display ? $email->display : $email->email;
				$emailTag = '<a href="mailto:' . $email->email . '">' . HtmlHelper::stringHTMLSafe($display) . '</a>';
				$emailAddress = $email->email;
				$emailDisplay = HtmlHelper::stringHTMLSafe($email->display);
			}
		}
		
		$template->addGlobalVar($field->fieldname, $emailTag);
		$template->addGlobalVar($field->fieldname.'_ADDRESS', $emailAddress);
		$template->addGlobalVar($field->fieldname.'_DISPLAY', $emailDisplay);
	}
	
	function _addVarHyperlink($field, $data, &$template)
	{
		$linkTitle = '';
		$linkTarget = '';
		$linkDisplay = '';
		
		if($data['VALUE'])
		{
			$settings = unserialize($field->settings);			
			$link = unserialize($data['VALUE']); 
			
			if($link->url)
			{
				$display 		= $link->display ? $link->display : $link->url;
				$linkTitle 		= $link->title;
				$linkTarget 	= $link->target;
				$linkDisplay 	= $link->display;
				$linkUrl 		=  $link->url;
				
				if($settings->outputMode == 0)
				{
					$linkTag = $link->url;
				}
				else
				{
					$linkTag = '<a href="' . $link->url . '" target="' . $link->target . '" title="' . $link->title . '">' . $display . '</a>';					
				}
				
				$template->addGlobalVar($field->fieldname, $linkTag);
			}
			else
			{
				$template->addGlobalVar($field->fieldname, '');
			}
			
			$template->addGlobalVar($field->fieldname.'_URL', $linkUrl);			
			$template->addGlobalVar($field->fieldname.'_TITLE', $linkTitle);			
			$template->addGlobalVar($field->fieldname.'_TARGET', $linkTarget);			
			$template->addGlobalVar($field->fieldname.'_DISPLAY', $linkDisplay);			
		}
	}

	function _addVarMultiSelectList($field, $data, &$template)
	{
		$tag = '';		
		$customFormat = ''; 
		
		if($data['VALUE'])
		{
			$settings 			= unserialize($field->settings);
			$selectedOptions 	= unserialize($data['VALUE']);
			
			if(count($selectedOptions) > 0)
			{
				$customFormat .= $settings->preListTag; 
				
				foreach($selectedOptions as $optionValue)
				{
					if(array_key_exists($optionValue, $settings->listOptions))
					{
						$tag .= '<li>' . htmlspecialchars($settings->listOptions[$optionValue]) . '</li>';
						$customFormat .= $settings->preElementTag.$settings->listOptions[$optionValue].$settings->postElementTag;		
					}
				}
				
				$customFormat .= $settings->postListTag; 				
			}			
		}
	
		$template->addGlobalVar($field->fieldname, $tag);		
		$template->addGlobalVar($field->fieldname.'_CUSTOM_FORMAT', $customFormat);
	}

	function _addVarInfoText($field, $data, &$template)
	{
		// Nothing to parse		
	}

	function _addVarDatePicker($field, $data, &$template)
	{
		$value = '';
		
		if($data['VALUE'])
		{
			$date = new JDate($data['VALUE']);
			$value= $date->toFormat($this->config->date_format);
		}

		$template->addGlobalVar($field->fieldname, $value);
	}

	function _addVarDisplayList($field, $data, &$template)
	{
		$output = '';
		
		if($data['VALUE'])
		{
			$settings = unserialize($field->settings);
			$values = unserialize($data['VALUE']);
						
			if($values)
			{
				foreach($values as $value)
				{
					$output .= '<li>'.htmlspecialchars($value).'</li>';
				}	
				
				if($settings->ordered)
				{
					$output = '<ol>'.$output.'</ol>';
				}
				else
				{
					$output = '<ul>'.$output.'</ul>';				
				}				
			}
		}
						
		$template->addGlobalVar($field->fieldname, $output);		
	}

	function _addVarFileUpload($field, $data, &$template)
	{
		if($data['VALUE'])
		{
			$settings = unserialize($field->settings);			
			$filename = $data['VALUE'];

			if($filename)
			{
				if($settings->outputMode == 0)
				{				
					$template->addGlobalVar($field->fieldname, Path::Combine(F2C_FileUpload::GetFileUrl($this->form->projectid, $this->form->id, $field->id), $filename));						
				}
				else
				{
					$template->addGlobalVar($field->fieldname, '<a href="'.Path::Combine(F2C_FileUpload::GetFileUrl($this->form->projectid, $this->form->id, $field->id), $filename).'" target="_blank">' . HtmlHelper::stringHTMLSafe($filename) . '</a>');
				}
				
				$template->addGlobalVar($field->fieldname.'_FILENAME', $filename);
				$template->addGlobalVar($field->fieldname.'_URL_RELATIVE', Path::Combine(F2C_FileUpload::GetFileUrl($this->form->projectid, $this->form->id, $field->id, true), $filename));
			}
			else
			{
				// no file was specified
				$template->addGlobalVar($field->fieldname, '');
				$template->addGlobalVar($field->fieldname.'_FILENAME', '');
				$template->addGlobalVar($field->fieldname.'_URL_RELATIVE', '');
			} 
		}			
	}

	function _addVarDatabaseLookup($field, $data, &$template)
	{
		$value = '';
		$text = '';
		
		if($data['VALUE'])
		{
			$settings 	= unserialize($field->settings);
			$value 		= $data['VALUE'];
			$text 		= '';
			$db 		=& JFactory::getDBO();			
			$db->setQuery($settings->query);				
			$rowList 	= $db->loadRowList(0);
			$text 		= $rowList[$value][1];
		}	

		$template->addGlobalVar($field->fieldname, $value);
		$template->addGlobalVar($field->fieldname.'_TEXT', $text);
	}
	
	function _addVarGeoCoder($field, $data, &$template)
	{
		if($data)
		{
			$template->addGlobalVar($field->fieldname.'_ADDRESS', HtmlHelper::stringHTMLSafe($data['ADDRESS']));
			$template->addGlobalVar($field->fieldname.'_LAT', $data['LAT']);
			$template->addGlobalVar($field->fieldname.'_LON', $data['LON']);
		}
		else
		{
			$template->addGlobalVar($field->fieldname.'_ADDRESS', '');
			$template->addGlobalVar($field->fieldname.'_LAT', '');
			$template->addGlobalVar($field->fieldname.'_LON', '');
		}
	}
	
	function _addVarDatabaseLookupMulti($field, $data, &$template)
	{
		$output 	= '';		
		$settings 	= unserialize($field->settings);
		$dicValues 	= array();		
		$db 		=& JFactory::getDBO();
		
		$db->setQuery($settings->query);				
		$dicValues = $db->loadRowList(0);

		if(count($data))
		{
			foreach($data['VALUE'] as $value)
			{
				$output .= '<li>'.$dicValues[$value][1].'</li>';
			}	
			
			if($settings->ordered)
			{
				$output = '<ol>'.$output.'</ol>';
			}
			else
			{
				$output = '<ul>'.$output.'</ul>';				
			}				
		}
						
		$template->addGlobalVar($field->fieldname, $output);		
	}	
	
	function getTemplateVars($formVars, &$usedVars)
	{	
		foreach($this->templates as $templateType => $template)
		{
			$this->_getTemplateVars($templateType, 'root', $formVars, $usedVars);
		}
	}
	
	function _getTemplateVars($templateType, $templateName, $formVars, &$usedVars)
	{
		$template = $this->templates[$templateType]->_templates[$templateName];
		
		// check which vars are used within the template
		foreach($formVars as $formVarAlias => $formVarName)
		{
			if(strpos($template['content'], '{'.$formVarAlias.'}') !== false)
			{
				$usedVars[] = strtoupper($formVarName);
			}
		}		
	
		// check dependent vars	
		if(array_key_exists('requiredvars', $template['attributes']))
		{
			if(count($template['attributes']['requiredvars']))
			{
				foreach($template['attributes']['requiredvars'] as $reqVarArray)
				{
					if(count($reqVarArray))
					{
						foreach($reqVarArray as $reqVar)
						{
							if($reqVar && $reqVar != $templateName)
							{
								$usedVars[] = $reqVar;
							}
						}
					}
				}
			}
		}
		
		if(count($template['dependencies']))
		{
			foreach($template['dependencies'] as $dependency)
			{
				$this->_getTemplateVars($templateType, $dependency, $formVars, $usedVars);
			}
		}
	}		
}
?>