<?php
defined('_JEXEC') or die('Restricted acccess');

class F2C_Renderer
{
	var $config;
	var $formId;
	var $translatedFields;
	var $contentTypeSettings;
	
	function F2C_Renderer($config, $formId, $translatedFields, $contentTypeSettings)
	{
		$this->config 				= $config;
		$this->formId 				= $formId;
		$this->translatedFields		= $translatedFields;
		$this->contentTypeSettings	= $contentTypeSettings;
	}
	
	function renderField($field, $fieldValues, $parms)
	{
		$F2C_FIELD_FUNCTION_MAPPING = array(F2C_FIELDTYPE_SINGLELINE => 'SingleLineText',
									F2C_FIELDTYPE_MULTILINETEXT => 'MultiLineText',
									F2C_FIELDTYPE_MULTILINEEDITOR => 'MultiLineEditor',
									F2C_FIELDTYPE_CHECKBOX => 'CheckBox',
									F2C_FIELDTYPE_SINGLESELECTLIST => 'SingleSelectList', 
									F2C_FIELDTYPE_IMAGE => 'Image',
									F2C_FIELDTYPE_IFRAME => 'IFrame', 
									F2C_FIELDTYPE_EMAIL => 'Email', 
									F2C_FIELDTYPE_HYPERLINK => 'Hyperlink', 
									F2C_FIELDTYPE_MULTISELECTLIST => 'MultiSelectList', 
									F2C_FIELDTYPE_INFOTEXT => 'InfoText', 
									F2C_FIELDTYPE_DATEPICKER => 'DatePicker', 
									F2C_FIELDTYPE_DISPLAYLIST => 'DisplayList', 
									F2C_FIELDTYPE_FILE => 'FileUpload', 
									F2C_FIELDTYPE_DATABASE_LOOKUP => 'DatabaseLookup', 
									F2C_FIELDTYPE_GEOCODER => 'GeoCoder',
									F2C_FIELDTYPE_DB_LOOKUP_MULTI => 'DatabaseLookupMulti');

		$fieldName 			= 't' . $field->id;
		$fieldDescription	= $this->_getFieldDescription($field);
		$functionName 		= '_render'.$F2C_FIELD_FUNCTION_MAPPING[$field->fieldtypeid];
				
		return $this->$functionName($field, $fieldName, $fieldValues, $fieldDescription, $parms);
	}
	
	function renderHiddenField($name, $value)
	{
		return '<input type="hidden" name="'.$name.'" id="'.$name.'" value="'.F2C_Renderer::stringHTMLSafe($value).'">';
	}
		
	function _renderSingleLineText($field, $fieldName, $fieldValues, $fieldDescription, $parms)
	{
		$html			= '';
		$value 			= $this->_getFieldValue($fieldValues, 'VALUE');
		$size			= $parms[0];
		$maxLength		= $parms[1];
		$attributes		= '';
		$fieldSettings 	= ($field->settings) ? unserialize($field->settings) : new F2C_SingleLineTextSettings(); 		
			
		if($fieldSettings->size) $size = $fieldSettings->size;
		if($fieldSettings->maxLength) $maxLength = $fieldSettings->maxLength;
		if($fieldSettings->attributes) $attributes = $fieldSettings->attributes;
		
		$html .= $this->_renderTextBox($fieldName, $value, $size, $maxLength, $attributes);
		
		if($fieldSettings->required && $this->contentTypeSettings->required_field_text)
		{
			$html .= '<span class="f2c_required">&nbsp;'.$this->contentTypeSettings->required_field_text.'</span>';
		}
		
		$html .= $fieldDescription;
		$html .= $this->renderHiddenField('hid'.$fieldName, $this->_getFieldContentId($fieldValues, 'VALUE'));
		
		return $html;
	}
	
	function _renderMultiLineText($field, $fieldName, $fieldValues, $fieldDescription, $parms)
	{
		$html 					= '';
		$fieldHtml 				= '';
		$attribs				= '';
		$fieldSettings 			= $field->settings ? unserialize($field->settings) : new F2C_MultiLineTextSettings();
		$value 					= $this->_getFieldValue($fieldValues, 'VALUE');
		
		if($fieldSettings->numRows || $fieldSettings->numCols || $fieldSettings->attributes)
		{
			if($fieldSettings->numRows) $attribs .= ' rows="'.$fieldSettings->numRows.'"';
			if($fieldSettings->numCols) $attribs .= ' cols="'.$fieldSettings->numCols.'"';
			if($fieldSettings->attributes) $attribs .= ' '.$fieldSettings->attributes;		
		}
		else
		{
			$attribs = $parms[0];
			$attribs .= ' class="text_area"';			
		}
		
		if($attribs)
		{
			$fieldHtml .= ' '.$attribs;
		}
		
		if($fieldSettings->maxNumChars != 0)
		{			
			if(function_exists('mb_substr_count') && function_exists('mb_substr') && function_exists('mb_strlen'))
			{
				$numNewLines = mb_substr_count($value, "\r\n", 'UTF-8');
				$charsRemaining = $fieldSettings->maxNumChars + $numNewLines - mb_strlen($value, 'UTF-8');			
				$fieldValue = mb_substr($value, 0, $fieldSettings->maxNumChars + $numNewLines, 'UTF-8');
			}
			else
			{
				$numNewLines = substr_count($value, "\r\n");
				$charsRemaining = $fieldSettings->maxNumChars + $numNewLines - strlen($value);			
				$fieldValue = substr($value, 0, $fieldSettings->maxNumChars + $numNewLines);
			}
			
			if($charsRemaining < 0)
			{
				$charsRemaining = 0;
			}
			
			$fieldHtml .= ' onKeyDown="F2C_limitTextArea(this.form.'.$fieldName.',this.form.'.$fieldName .'remLen,'.$fieldSettings->maxNumChars.');" onKeyUp="F2C_limitTextArea(this.form.' . $fieldName . ',this.form.'.$fieldName .'remLen,'.$fieldSettings->maxNumChars.');"';
		}
		
		$html .= '<textarea name="'.$fieldName.'" id="'.$fieldName.'"'.$fieldHtml.'>'.$value.'</textarea>';
		
		if($fieldSettings->maxNumChars != 0)
		{
			$html .= '<br/><input readonly type="text" name="'.$fieldName .'remLen" size="6" maxlength="6" value="'.$charsRemaining.'"> '.Jtext::_('CHARACTERS_LEFT');		
		}
		
		$html .= $this->_renderRequiredText($fieldSettings);														
		$html .= $fieldDescription;
		$html .= $this->renderHiddenField('hid'.$fieldName, $this->_getFieldContentId($fieldValues, 'VALUE'));
		
		return $html;
	}
	
	function _renderMultiLineEditor($field, $fieldName, $fieldValues, $fieldDescription, $parms)
	{
		$editor			=& JFactory::getEditor();
		$fieldSettings 	= $field->settings ? unserialize($field->settings) : new F2C_MultiLineEditorSettings();		
		$value 			= $this->_getFieldValue($fieldValues, 'VALUE');

		$width	= $parms[0];
		$height = $parms[1];
		$col	= $parms[2];
		$row	= $parms[3];
		
		if($field->settings)
		{
			$settings = unserialize($field->settings);
			if($settings->numRows || $settings->numCols || $settings->height || $settings->width)
			{
				$width	= $settings->width;
				$height = $settings->height;
				$col	= $settings->numCols;
				$row	= $settings->numRows;
			}
		}
		
		$html = $editor->display($fieldName, $value, $width, $height, $col, $row);
		$html .= $this->_renderRequiredText($fieldSettings);																
		$html .= $fieldDescription;
		$html .= $this->renderHiddenField('hid'.$fieldName, $this->_getFieldContentId($fieldValues, 'VALUE'));
		
		return $html;
	}
	
	function _renderCheckBox($field, $fieldName, $fieldValues, $fieldDescription, $parms)
	{
		$html			= '';
		$value			= $this->_getFieldValue($fieldValues, 'VALUE');
		$fieldSettings 	= ($field->settings) ? unserialize($field->settings) : new F2C_CheckBoxSettings(); 		
				
		$html 	.= '<input type="checkbox" name="'.$fieldName.'" id="'.$fieldName.'" '.$fieldSettings->attributes.' value="true"'.(($value) ? ' checked' : '') . '>';		
		$html 	.= $this->_renderRequiredText($fieldSettings);
		$html 	.= $fieldDescription;
		$html 	.= $this->renderHiddenField('hid'.$fieldName, $this->_getFieldContentId($fieldValues, 'VALUE'));
		
		return $html;
	}
	
	function _renderSingleSelectList($field, $fieldName, $fieldValues, $fieldDescription, $parms)
	{ 
		$html			= '';
		$fieldValue		= $this->_getFieldValue($fieldValues, 'VALUE');		
		$listOptions 	= null;
		$fieldSettings	= unserialize($field->settings);

		if($fieldSettings->showEmptyText)
		{
			$listOptions[] = JHTMLSelect::option('', F2C_Renderer::stringHTMLSafe($fieldSettings->emptyText));
		}
			      				
		if(count($fieldSettings->listOptions))
		{					
			foreach($fieldSettings->listOptions as $key=>$value)
			{
				$listOptions[] = JHTMLSelect::option($key, F2C_Renderer::stringHTMLSafe($value));  	
			}
			
		}
		
		if($fieldSettings->displayMode == 0)
		{
			$html .= JHTMLSelect::genericlist($listOptions, $fieldName, $fieldSettings->attributes, 'value', 'text', $fieldValue);
		}
		else
		{  
			$html .= JHTMLSelect::radioList($listOptions, $fieldName, $fieldSettings->attributes, 'value', 'text', $fieldValue);
		}
		
		$html .= $this->_renderRequiredText($fieldSettings);		
		$html .= $fieldDescription;
		$html .= $this->renderHiddenField('hid'.$fieldName, $this->_getFieldContentId($fieldValues, 'VALUE'));
		
		return $html;
	}

	function _renderImage($field, $fieldName, $fieldValues, $fieldDescription, $parms)
	{
		$html				= '';
		$fieldValue			= $this->_getFieldValue($fieldValues, 'VALUE');
		$image 				= $fieldValue ? unserialize($fieldValue) : new F2C_Image();		
		$fieldSettings		= $field->settings ? unserialize($field->settings) : new F2C_ImageSettings();		
		$uploadAttribs 		= $fieldSettings->uploadAttributes ? $fieldSettings->uploadAttributes : 'class="inputbox"';
		$deleteAttribs 		= $fieldSettings->deleteAttributes ? $fieldSettings->deleteAttributes : 'class="inputbox"';		
		$widthAltText		= $fieldSettings->altValueAttributes ? '' : $parms[0];
		$maxLengthAltText	= $fieldSettings->altValueAttributes ? '' : $parms[1];
		$widthTitle			= $fieldSettings->titleAttributes ? '' : $parms[0];
		$maxLengthTitle		= $fieldSettings->titleAttributes ? '' : $parms[1];
		
		$html .= '<table><tr><td>&nbsp;</td><td>';
		$html .= '<input type="file" id="'.$fieldName.'_fileupload" name="'.$fieldName.'_fileupload" '.$uploadAttribs.'>&nbsp;';
		$html .= '<input type="button" onclick="clearUpload(\''.$fieldName.'_fileupload\');return false;" value="'.JText::_('CLEAR_FIELD').'" />&nbsp;';
		
		// No need for a delete check box when the field is required
		if(!$fieldSettings->required)
		{
			$html .= '<input type="checkbox" id="'.$fieldName.'_del" name="'.$fieldName.'_del" '.$deleteAttribs.'>&nbsp;'.JText::_('DELETE_IMAGE');
		}
		
		$html .= $this->_renderRequiredText($fieldSettings);										
		$html .= F2C_Renderer::renderHiddenField($fieldName . '_filename', $image->filename);
		$html .= $fieldDescription;
		
		$html .= '</td></tr>';
		
		if($fieldSettings->showAltTag)
		{
			$html .= '<tr><td>'.JText::_('ALT_TEXT').':</td>';
			$html .= '<td>'.$this->_renderTextBox($fieldName.'_alt', $image->alt, $widthAltText, $maxLengthAltText, $fieldSettings->altValueAttributes).'</td>';
		}
		else
		{
			$html .= F2C_Renderer::renderHiddenField($fieldName.'_alt', $image->alt);
		}
		
		if($fieldSettings->showTitleTag)
		{
			$html .= '</tr><tr><td>'.JText::_('TITLE').':</td>';
			$html .= '<td>'.$this->_renderTextBox($fieldName.'_title', $image->title, $widthTitle, $maxLengthTitle, $fieldSettings->titleAttributes).'</td></tr>';
		}
		else
		{
			$html .= F2C_Renderer::renderHiddenField($fieldName.'_title', $image->title);
		}

		$html .= '<tr><td valign="top">'.JText::_('PREVIEW').':</td><td>';

		if($image->filename)
		{
			$thumbSrc = Path::Combine(F2C_Image::GetThumbnailsUrl($field->projectid, $this->formId), $image->CreateThumbnailImageName($image->filename, $field->id));
			$html .= '<img id="'.$fieldName.'_preview" src="' . $thumbSrc . '" style="border: 1px solid #000000;">';
		}

		$html .= '</td></tr></table>';
		$html .= $this->renderHiddenField('hid'.$fieldName, $this->_getFieldContentId($fieldValues, 'VALUE'));
		
		return $html;		
	}
	
	function _renderMultiSelectList($field, $fieldName, $fieldValues, $fieldDescription, $parms)
	{
		$html			= '';
		$valueList 		= array();
		$fieldValue		= $this->_getFieldValue($fieldValues, 'VALUE');		
		$fieldSettings 	= unserialize($field->settings);
		
		$html .= '<table><tr><td>';

		if($fieldValue)
		{
			$fieldValue = unserialize($fieldValue);
			if(count($fieldValue) > 0)
			{
				foreach($fieldValue as $value)
				{
					$valueList[$value] = $value;
				}
			}
		}	
		
		foreach($fieldSettings->listOptions as $optionKey => $optionValue)
		{
			$html .= '<input type=checkbox name="' . $fieldName . '[]" value="' . F2C_Renderer::stringHTMLSafe($optionKey) . '" '.$fieldSettings->attributes;
			
			if(array_key_exists($optionKey, $valueList))
			{
				$html .= ' checked';
			}
			
			$html .= '>' . F2C_Renderer::stringHTMLSafe($optionValue) . '<br/>';
		}

		$html .= '</td><td valign="top">';
		$html .= $this->_renderRequiredText($fieldSettings);												
		$html .= $fieldDescription.'</td></tr></table>';
		$html .= $this->renderHiddenField('hid'.$fieldName, $this->_getFieldContentId($fieldValues, 'VALUE'));
						
		return $html;
	}
	
	function _renderIFrame($field, $fieldName, $fieldValues, $fieldDescription, $parms)
	{
		$value 				= $this->_getFieldValue($fieldValues, 'VALUE');
		$iframe 			= $value ? unserialize($value) : new F2C_IFrame();
		$fieldSettings 		= $field->settings ? unserialize($field->settings) : new F2C_IFrameSettings();				
		$html 				= '';
				
		$html .= '<table><tr>';
		$html .= '<td>'.JText::_('URL').':</td>';
		$html .= '<td>'.$this->_renderTextBox($fieldName, $iframe->url, 65, 75, $fieldSettings->iFrameAttributes);								
		$html .= $this->_renderRequiredText($fieldSettings);								
		$html .= $fieldDescription.'</td>';
		$html .= '</tr><tr>';
		$html .= '<td>'.JText::_('WIDTH').':</td>';
		$html .= '<td>'.$this->_renderTextBox($fieldName.'_width', $iframe->width, 5, 4, $fieldSettings->widthAttributes).'&nbsp;';		      							
		$html .= JText::_('HEIGHT').':&nbsp;';
		$html .= $this->_renderTextBox($fieldName.'_height', $iframe->height, 5, 4, $fieldSettings->heightAttributes).'</td>';		      							
		$html .= '</tr></table>';
		$html .= $this->renderHiddenField('hid'.$fieldName, $this->_getFieldContentId($fieldValues, 'VALUE'));
		
		return $html;	
	}
			
	function _renderEmail($field, $fieldName, $fieldValues, $fieldDescription, $parms)
	{
		$html 				= '';
		$value 				= $this->_getFieldValue($fieldValues, 'VALUE');
		$email 				= $value ? unserialize($value) : new F2C_Email();
		$fieldSettings 		= ($field->settings) ? unserialize($field->settings) : new F2C_EmailSettings(); 		
		
		$html .= '<table><tr>';
		$html .= '<td>'.JText::_('EMAIL').':</td>';
		$html .= '<td>'.$this->_renderTextBox($fieldName, $email->email, 40, 100, $fieldSettings->emailAttributes);
		$html .= $this->_renderRequiredText($fieldSettings);
		$html .= $fieldDescription.'</td>';
		$html .= '</tr>';
		
		if($fieldSettings->showDisplayAs)
		{
			$html .= '<tr>';
			$html .= '<td>'.JText::_('DISPLAY_AS').':</td>';
			$html .= '<td>'.$this->_renderTextBox($fieldName.'_display', $email->display, 40, 100, $fieldSettings->displayAsAttributes).'</td>';
			$html .= '</tr>';
		}
				
		$html.= '</table>';
		
		if(!$fieldSettings->showDisplayAs)
		{
			$this->renderHiddenField($fieldName.'_display', $email->display);
		}
		
		$html .= $this->renderHiddenField('hid'.$fieldName, $this->_getFieldContentId($fieldValues, 'VALUE'));
		
		return $html;
	}
	
	function _renderHyperlink($field, $fieldName, $fieldValues, $fieldDescription, $parms)
	{
		$html 				= '';
		$fieldSettings		= $field->settings ? unserialize($field->settings) : new F2C_LinkSettings();
		$value 				= $this->_getFieldValue($fieldValues, 'VALUE');
		$link 				= $value ? unserialize($value) : new F2C_Link();
		$listTarget[] 		= JHTMLSelect::option('_top', 'Parent window');
		$listTarget[] 		= JHTMLSelect::option('_blank','New window');	
	
		$html .= '<table><tr><td>'.JText::_('URL').':</td><td>';
		$html .= $this->_renderTextBox($fieldName, $link->url, 40, 300, $fieldSettings->linkAttributes); 
		$html .= $this->_renderRequiredText($fieldSettings);								
		$html .= $fieldDescription;
		$html .= '</td>';
		$html .= '</tr>';
		
		if($fieldSettings->showDisplayAs)
		{
			$html .= '<tr>';
			$html .= '<td>'.JText::_('DISPLAY_AS').':</td>';
			$html .= '<td>'.$this->_renderTextBox($fieldName.'_display', $link->display, 40, 100, $fieldSettings->displayAsAttributes).'</td>';
			$html .= '</tr>';			
		}
		
		if($fieldSettings->showTitle)
		{
			$html .= '<tr>';
			$html .= '<td>'.JText::_('TITLE').':</td>';
			$html .= '<td>'.$this->_renderTextBox($fieldName.'_title', $link->title, 40, 100, $fieldSettings->titleAttributes).'</td>';		      							
			$html .= '</tr>';
		}

		if($fieldSettings->showTarget)
		{
			$html .= '<tr>';
			$html .= '<td>'.JText::_('TARGET').':</td>';	      							
			$html .= '<td>'.JHTMLSelect::genericlist($listTarget, $fieldName . '_target',$fieldSettings->targetAttributes,'value', 'text', $link->target).'</td>';
			$html .= '</tr>';
		}
		
		$html .= '</table>';
		
		if(!$fieldSettings->showDisplayAs)
		{
			$html .= $this->renderHiddenField($fieldName.'_display', '');
		}
		
		if(!$fieldSettings->showTitle)
		{
			$html .= $this->renderHiddenField($fieldName.'_title', '');
		}
		
		if(!$fieldSettings->showTarget)
		{
			$html .= $this->renderHiddenField($fieldName.'_target', '');
		}
		
		$html .= $this->renderHiddenField('hid'.$fieldName, $this->_getFieldContentId($fieldValues, 'VALUE'));

		return $html;
	}
		
	function _renderDatePicker($field, $fieldName, $fieldValues, $fieldDescription, $parms)
	{
		$html 			= '';
		$value 			= $this->_getFieldValue($fieldValues, 'VALUE');
		$fieldSettings 	= $field->settings ? unserialize($field->settings) : new F2C_DatePickerSettings();
		$attributes 	= $fieldSettings->attributes ? $fieldSettings->attributes : 'class="inputbox"';
		if($value)
		{
			$date = new JDate($value);
			$value = $date->toFormat($this->config->date_format);
		}
	      				
		$html .= '<input '.$attributes.' type="text" name="'.$fieldName.'" id="'.$fieldName.'" size="10" maxlength="19" value="'.$value.'" />';
		$html .= '<a href="#" onclick="return showCalendar(\''.$fieldName.'\', \''.$this->config->date_format.'\');"><img class="calendar" src="images/blank.png" alt="calendar" /></a>';
		
		$html .= $this->_renderRequiredText($fieldSettings);
		$html .= $fieldDescription;
		$html .= $this->renderHiddenField('hid'.$fieldName, $this->_getFieldContentId($fieldValues, 'VALUE'));
		
		return $html;
	}
	
	function _renderInfoText($field, $fieldName, $fieldValues, $fieldDescription, $parms)
	{
	    $infoText = $field->settings ? unserialize($field->settings) : new F2C_InfoTextSettings();
		
	    return $infoText->text . $fieldDescription;		
	}

	function _renderDisplayList($field, $fieldName, $fieldValues, $fieldDescription, $parms)
	{
		$html 				= '';
		$fieldValue 		= $this->_getFieldValue($fieldValues, 'VALUE');
		$fieldSettings		= $field->settings ? unserialize($field->settings) : new F2C_DisplayListSettings();
		$attributesTable	= $fieldSettings->attributesTable ? $fieldSettings->attributesTable : 'border="1"';
		
		$html .= '<table><tr><td>';
		
		$html .= '<table '.$attributesTable.' id="'.$fieldName.'" cellspacing="0" cellpadding="0">
					<tr '.$fieldSettings->attributesTr.'>
						<th '.$fieldSettings->attributesTh.' style="width:200px;">' . JText::_('LIST_ITEM') . '</th>
						<th '.$fieldSettings->attributesTh.' style="width:41px;">&nbsp;</th>	
						<th '.$fieldSettings->attributesTh.' style="width:26px;">&nbsp;</th>			
						<th '.$fieldSettings->attributesTh.' style="width:26px;">&nbsp;</th>						
					</tr>';
									
		$rowcount = 0;
		$imgPath = JURI::root(true).'/components/com_form2content/media/images/';
						
		if($fieldValue)
		{
			$fieldValue = unserialize($fieldValue);
		
			if(count($fieldValue) > 0)
			{
				foreach($fieldValue as $value)
				{
					$rowId = $fieldName.'_'.$rowcount;
					$rowcount++;
					$html .= '<tr id="'.$rowId.'" '.$fieldSettings->attributesTr.'>
							  <td '.$fieldSettings->attributesTd.'>
							  	<input type="hidden" name="'.$fieldName.'RowKey[]" value="'.$rowId.'"/>
							  	<input type="text" id="'.$rowId.'val" name="'.$rowId.'val" size="40" value="' . htmlspecialchars($value) . '" maxlength="255" '.$fieldSettings->attributesItemText.' />
							  </td>
							  <td '.$fieldSettings->attributesTd.'><a href="javascript:moveUp(\''.$fieldName.'\',\''.$rowId.'\');"><img src="'.$imgPath.'uparrow.png" alt="' . JText::_('UP') . '" /></a><a href="javascript:moveDown(\''.$fieldName.'\',\''.$rowId.'\');"><img src="'.$imgPath.'downarrow.png" alt="' . JText::_('DOWN') . '" /></a></td>
							  <td '.$fieldSettings->attributesTd.'><a href="javascript:removeRow(\''.$rowId.'\');"><img src="'.$imgPath.'remove.png" alt="' . JText::_('DELETE') . '" /></a></td>
							  <td '.$fieldSettings->attributesTd.'><a href="javascript:addDisplayListRow(\''.$fieldName.'\',\''.$rowId.'\');"><img src="'.$imgPath.'add.png" alt="' . JText::_('ADD') . '" /></a></td>
							  </tr>';
				}
			}
		}
		
		$html .= 	'</table>
					 <br/>
					 <input type="button" value="' . JText::_('ADD_LIST_ITEM') . '" '.$fieldSettings->attributesAddButton.' onclick="addDisplayListRow(\''.$fieldName.'\',\'\');" />
					 <input type="hidden" name="'.$fieldName.'MaxKey" id="'.$fieldName.'MaxKey" value="'.$rowcount.'"/>';
					 		
		$html .= '</td><td valign="top">';
		
		$html .= $this->_renderRequiredText($fieldSettings);
		$html .= $fieldDescription.'</td></tr></table>';
		$html .= $this->renderHiddenField('hid'.$fieldName, $this->_getFieldContentId($fieldValues, 'VALUE'));
		
		return $html;		
	}	
	
	function _renderFileUpload($field, $fieldName, $fieldValues, $fieldDescription, $parms)
	{
		$html				= '';
		$fieldValue			= $this->_getFieldValue($fieldValues, 'VALUE');
		$fieldSettings		= $field->settings ? unserialize($field->settings) : new F2C_FileSettings();
		$uploadAttributes	= $fieldSettings->uploadAttributes ? $fieldSettings->uploadAttributes : 'class="inputbox"';
		$deleteAttributes	= $fieldSettings->deleteAttributes ? $fieldSettings->deleteAttributes : 'class="inputbox"';
		
		$html .= '<table><tr><td>';
		$html .= '<input type="file" id="'.$fieldName.'_fileupload" name="'.$fieldName.'_fileupload" '.$uploadAttributes.'>&nbsp;';
		$html .= '<input type="button" onclick="clearUpload(\''.$fieldName.'_fileupload\');return false;" value="'.JText::_('CLEAR_FIELD').'" />&nbsp;';		
		$html .= '<input type="checkbox" id="'.$fieldName.'_del" name="'.$fieldName.'_del" '.$deleteAttributes.'">&nbsp;'.JText::_('DELETE_FILE');
		$html .= $this->_renderRequiredText($fieldSettings);								
		$html .= $this->renderHiddenField($fieldName . '_filename', $fieldValue);		
		$html .= $fieldDescription;
		$html .= '</td></tr><tr><td valign="top">'.JText::_('PREVIEW').':&nbsp;';
				
		if($fieldValue)
		{
			$html .= '<a id="'.$fieldName.'_preview" href="'.Path::Combine(F2C_FileUpload::GetFileUrl($field->projectid, $this->formId, $field->id), $fieldValue).'" target="_blank">' . F2C_Renderer::stringHTMLSafe($fieldValue) . '</a>';
		}
				
		$html .= '</td></tr></table>';
		$html .= $this->renderHiddenField('hid'.$fieldName, $this->_getFieldContentId($fieldValues, 'VALUE'));
			
		return $html;
		
	}
	
	function _renderDatabaseLookup($field, $fieldName, $fieldValues, $fieldDescription, $parms)
	{
		$html				= '';
		$listOptions 		= array();
		$fieldSettings		= unserialize($field->settings);
		$fieldValue			= $this->_getFieldValue($fieldValues, 'VALUE');

		if($fieldSettings->showEmptyText)
		{
			$listOptions[] = JHTMLSelect::option('', F2C_Renderer::stringHTMLSafe($fieldSettings->emptyText),'key','value');
		}
			      				
		$db =& JFactory::getDBO();
		$db->setQuery($fieldSettings->query);
		$rowList = $db->loadRowList();

		if(count($rowList))
		{
			foreach($rowList as $row)
			{
				$listOptions[] = JHTMLSelect::option($row[0], F2C_Renderer::stringHTMLSafe($row[1]),'key','value');
			}
		}

		if($fieldSettings->displayMode == 0)
		{
			$html .= JHTMLSelect::genericlist($listOptions, $fieldName, $fieldSettings->attributes, 'key', 'value', $fieldValue);
		}
		else
		{  
			$html .= JHTML::_('select.radiolist', $listOptions, $fieldName, $fieldSettings->attributes, 'key', 'value', $fieldValue);	
		}
		
		$html .= $this->_renderRequiredText($fieldSettings);
		$html .= $fieldDescription;
		$html .= $this->renderHiddenField('hid'.$fieldName, $this->_getFieldContentId($fieldValues, 'VALUE'));
		
		return $html;
	}
	
	function _renderGeoCoder($field, $fieldName, $fieldValues, $fieldDescription, $parms)
	{
		$fieldSettings		= unserialize($field->settings);		
		$defaultLat			= ($fieldSettings->mapLat != '') ? $fieldSettings->mapLat : 55.166085;
		$defaultLon			= ($fieldSettings->mapLon != '') ? $fieldSettings->mapLon : 10.712890;
		$mapWidth 			= ($fieldSettings->mapWidth) ? $fieldSettings->mapWidth : 350;
		$mapHeight 			= ($fieldSettings->mapHeight) ? $fieldSettings->mapHeight : 350;
		$attributesAddress 	= $fieldSettings->attributesAddress ? $fieldSettings->attributesAddress : 'class="inputbox"';	
		$attributesLookup 	= $fieldSettings->attributesLookup ? $fieldSettings->attributesLookup : 'class="inputbox"';	
		$attributesClear 	= $fieldSettings->attributesClear ? $fieldSettings->attributesClear : 'class="inputbox"';			
		$addressValue 		= '';
		$addressId 			= '';
		$latValue 			= '';
		$latId 				= '';
		$lonValue 			= '';
		$lonId 				= '';
		
		if($fieldValues)
		{
			if(array_key_exists('ADDRESS', $fieldValues))
			{
				$addressValue = $fieldValues['ADDRESS']->content;
				$addressId = $fieldValues['ADDRESS']->id;
			}
	
			if(array_key_exists('LAT', $fieldValues))
			{
				$latValue = $fieldValues['LAT']->content;
				$latId = $fieldValues['LAT']->id;
			}
	
			if(array_key_exists('LON', $fieldValues))
			{
				$lonValue = $fieldValues['LON']->content;
				$lonId = $fieldValues['LON']->id;
			}
		}
		
		$latLonDisplay = ($latValue && $lonValue) ? '('.$latValue.', '.$lonValue.')' : '';
				
		if($fieldSettings->showMap)
		{
			$document =& JFactory::getDocument();
		
			$latOnMap = $latLonDisplay ? $latValue : $defaultLat;
			$lonOnMap = $latLonDisplay ? $lonValue : $defaultLon;
		
			$js =	'window.addEvent(\'load\', function() {
					 var latlng = new google.maps.LatLng('.$latOnMap.', '.$lonOnMap.');
    				 var myOptions = { zoom: '.$fieldSettings->mapZoom.', center: latlng, mapTypeId: google.maps.MapTypeId.'.$fieldSettings->mapType.' }; ' .
    				$fieldName.'_map = new google.maps.Map(document.getElementById("'.$fieldName.'_map_canvas"), myOptions); ';

			if($latLonDisplay)
			{
				// initialize the marker
				$js .= 'eval('.$fieldName.'_marker = new google.maps.Marker({ map: '.$fieldName.'_map, position: latlng }));';
			}

			$js .=	' });';
			  		
			$document->addScriptDeclaration($js);
		}
										
		$html = '';
		
		if($fieldSettings->showMap)
		{
			$html .= '<div id="'.$fieldName.'_map_canvas" style="width: '.$mapWidth.'px; height: '.$mapHeight.'px;"></div><br/>';
		}
		
 		$html .= '<table>';
 		$html .= '<tr><td>'.JText::_('ADDRESS_OF_LOCATION').': </td><td><input id="'.$fieldName.'_address" name="'.$fieldName.'_address" type="textbox" '.$attributesAddress.' value="'.$this->stringHTMLSafe($addressValue).'" style="width:300px;">';
 		$html .= '&nbsp;<input type="button" '.$attributesLookup.' value="'.JText::_('LOOKUP_LAT_LON').'" onclick="F2C_GeoCoderConvertAddress(\''.$fieldName.'\');">';
 		$html .= '&nbsp;<input type="button" '.$attributesClear.' value="'.JText::_('CLEAR_RESULTS').'" onclick="F2C_GeoCoderClearResults(\''.$fieldName.'\');">';
		$html .= $this->_renderRequiredText($fieldSettings); 		
 		$html .= $fieldDescription;
 		$html .= $this->renderHiddenField($fieldName.'_hid_lat', $latValue);
 		$html .= $this->renderHiddenField($fieldName.'_hid_lon', $lonValue);
 		$html .= '</td></tr>';
 		$html .= '<tr><td>'.JText::_('LAT_LON').': </td><td><span id="'.$fieldName.'_latlon" name="'.$fieldName.'_latlon">'.$latLonDisplay.'</span>';
 		$html .= '<span id="'.$fieldName.'_error" name="'.$fieldName.'_error" style="display: none;">'.JText::_('ERROR_GEOCODER_PROCESS').'</span></td></tr>';
 		$html .= '</table>';
 		$html .= $this->renderHiddenField('hid'.$fieldName.'_lat', $latId);
 		$html .= $this->renderHiddenField('hid'.$fieldName.'_lon', $lonId);
 		$html .= $this->renderHiddenField('hid'.$fieldName.'_address', $addressId);

		return $html;
	}
	
	function _renderDatabaseLookupMulti($field, $fieldName, $fieldValues, $fieldDescription, $parms)
	{
		$html 				= '';
		$fieldSettings		= $field->settings ? unserialize($field->settings) : new F2C_DatabaseLookupMultiSettings();
		$attributesTable	= $fieldSettings->attributesTable ? $fieldSettings->attributesTable : 'border="1"';
		$valueList			= array();
		$listOptions		= array();

		// Prepare drop down list
		if($fieldSettings->showEmptyText)
		{
			$listOptions[] = JHTMLSelect::option('', F2C_Renderer::stringHTMLSafe($fieldSettings->emptyText),'key','value');
		}
		
		$db =& JFactory::getDBO();
		$db->setQuery($fieldSettings->query);
		$rowList = $db->loadRowList(0);

		if(count($rowList))
		{
			foreach($rowList as $row)
			{
				$listOptions[] = JHTMLSelect::option($row[0], F2C_Renderer::stringHTMLSafe($row[1]),'key','value');
			}
		}
		
		$html .= '<table><tr><td>';
		
		$html .= '<table '.$attributesTable.' id="'.$fieldName.'" cellspacing="0" cellpadding="0">		
					<tr '.$fieldSettings->attributesTr.'>
						<th '.$fieldSettings->attributesTh.' style="width:200px;">' . JText::_('LIST_ITEM') . '</th>
						<th '.$fieldSettings->attributesTh.' style="width:41px;">&nbsp;</th>	
						<th '.$fieldSettings->attributesTh.' style="width:26px;">&nbsp;</th>			
					</tr>';
		
		$rowcount = 0;
		$imgPath = JURI::root(true).'/components/com_form2content/media/images/';
						
		if(count($fieldValues))
		{
			$valueList = $fieldValues['VALUE'];
		}
		
		if(count($valueList))
		{
			foreach($valueList as $valueListItem)
			{
				$value = $valueListItem->content;
				
				$rowId = $fieldName.'_'.$rowcount;
				$rowcount++;
				$html .= '<tr id="'.$rowId.'" '.$fieldSettings->attributesTr.'>
						  <td '.$fieldSettings->attributesTd.'>
						  	<input type="hidden" name="'.$fieldName.'RowKey[]" value="'.$rowId.'"/>
						  	<input type="hidden" id="'.$rowId.'val" name="'.$rowId.'val" value="' . htmlspecialchars($value) . '" />'.
							F2C_Renderer::stringHTMLSafe($rowList[$value][1]).'
						  </td>
						  <td '.$fieldSettings->attributesTd.'><a href="javascript:moveUp(\''.$fieldName.'\',\''.$rowId.'\');"><img src="'.$imgPath.'uparrow.png" alt="' . JText::_('UP') . '" /></a><a href="javascript:moveDown(\''.$fieldName.'\',\''.$rowId.'\');"><img src="'.$imgPath.'downarrow.png" alt="' . JText::_('DOWN') . '" /></a></td>
						  <td '.$fieldSettings->attributesTd.'><a href="javascript:removeRow(\''.$rowId.'\');"><img src="'.$imgPath.'remove.png" alt="' . JText::_('DELETE') . '" /></a></td>
						  </tr>';
			}
		}
		
		$html .= 	'</table>
					 <br/>				
					 <input type="hidden" name="'.$fieldName.'MaxKey" id="'.$fieldName.'MaxKey" value="'.$rowcount.'"/>';
					 		
		$html .= '</td><td valign="top">';
		
		$html .= $this->_renderRequiredText($fieldSettings);
		$html .= $fieldDescription.'</td></tr></table>';
		$html .= $this->renderHiddenField('hid'.$fieldName, $field->id);
		
		$html .= JHTMLSelect::genericlist($listOptions, $fieldName.'_lookup', $fieldSettings->attributesSelect, 'key', 'value', '').'&nbsp;';
		$html .= '<input type="button" value="' . JText::_('ADD') . '" '.$fieldSettings->attributesAddButton.' onclick="addDbLookupkMultiRow(\''.$fieldName.'\',\'\');" />';
		
		return $html;		
	}	
	
	function _renderTextBox($name, $value = '', $size = '', $maxlength = '', $tags = '')
	{
		$class = ($tags) ? '' : 'class="inputbox"';
		
		$html = '<input type="text" '.$class.' name="'.$name.'" id="'.$name.'"';
		$html .= ($value != '') ? ' value= "' . F2C_Renderer::stringHTMLSafe($value) . '"' : '';
		$html .= $size ? ' size= "' . $size . '"' : '';
		$html .= $maxlength ? ' maxlength= "' . $maxlength . '"' : '';
		$html .= $tags . '/>';
		
		return $html;
	}
	
	function _detectUTF8($string)
	{
	    return preg_match('%(?:
	        [\xC2-\xDF][\x80-\xBF]             # non-overlong 2-byte
	        |\xE0[\xA0-\xBF][\x80-\xBF]        # excluding overlongs
	        |[\xE1-\xEC\xEE\xEF][\x80-\xBF]{2} # straight 3-byte
	        |\xED[\x80-\x9F][\x80-\xBF]        # excluding surrogates
	        |\xF0[\x90-\xBF][\x80-\xBF]{2}     # planes 1-3
	        |[\xF1-\xF3][\x80-\xBF]{3}         # planes 4-15
	        |\xF4[\x80-\x8F][\x80-\xBF]{2}     # plane 16
	        )+%xs', 
	    $string);
	}

	function stringHTMLSafe($string)
	{
		if(F2C_Renderer::_detectUTF8($string))
		{
			$safeString = htmlentities($string, ENT_COMPAT, 'UTF-8');
		}
		else
		{
			$safeString = htmlentities($string, ENT_COMPAT);
		}
		
		return $safeString;
	}
	
	function _getFieldValue($fieldValues, $attribute, $default = '')
	{
		return ($fieldValues && array_key_exists($attribute, $fieldValues)) ? $fieldValues[$attribute]->content : $default;
	}

	function _getFieldContentId($fieldValues, $attribute)
	{
		return ($fieldValues && array_key_exists($attribute, $fieldValues)) ? $fieldValues[$attribute]->id : 0;
	}

	function _getFieldDescription($field)
	{
		$fieldLabel = '';
		$fieldDescription = '';
		
		if(array_key_exists($field->id, $this->translatedFields))
		{
			$fieldLabel 		= $this->translatedFields[$field->id]->title_translation;
			$fieldDescription 	= $this->translatedFields[$field->id]->description_translation;
		}
		else
		{
			$fieldLabel = $field->title;
			$fieldDescription = $field->description;				
		}
		
		if($fieldDescription)
		{
			$fieldDescription = '&nbsp;' . JHTML::tooltip($fieldDescription, $fieldLabel);				
		}
		
		return $fieldDescription;		
	}
	
	function renderFieldLabel($field)
	{
		$label = '';
		
		if($field->fieldtypeid != F2C_FIELDTYPE_INFOTEXT)
		{ 
			$labelText = (array_key_exists($field->id, $this->translatedFields)) ? $this->translatedFields[$field->id]->title_translation : $field->title; 
			$label = '<label for="t'.$field->id.'">'.$labelText.':</label>';
		}
		
		return $label; 
	}
	
	function _renderRequiredText($fieldSettings)
	{
		if($fieldSettings->required && $this->contentTypeSettings->required_field_text)
		{
			return '<span class="f2c_required">&nbsp;'.$this->contentTypeSettings->required_field_text.'</span>';
		}		
	}
}
?>